@@ -42,7 +42,6 @@ pomelo! {
     %type identifier visitors::Identifier<T::Symbol>;
 
     %type bound_symbol T::Symbol;
-    %type bound_symbols Vec<T::Symbol>;
     %type fresh_symbol T::Symbol;
     %type fresh_symbols Vec<T::Symbol>;
     %type any_symbol T::Symbol;
@@ -93,12 +92,10 @@ pomelo! {
     %start_symbol command;
 
     bound_symbol ::= Symbol(s) { extra.0.visit_bound_symbol(s)? }
-    fresh_symbol ::= Symbol(s) { extra.0.visit_fresh_symbol(s)? }
+    fresh_symbol ::= Symbol(s) { extra.0.visit_fresh_symbol(s, crate::visitors::SymbolKind::Unknown)? }
     any_symbol ::= Symbol(s) { extra.0.visit_any_symbol(s)? }
     keyword ::= Keyword(s) { extra.0.visit_keyword(s)? }
 
-    bound_symbols ::= bound_symbol(x) { vec![x] }
-    bound_symbols ::= bound_symbols(mut xs) bound_symbol(x) { xs.push(x); xs }
     fresh_symbols ::= fresh_symbol(x) { vec![x] }
     fresh_symbols ::= fresh_symbols(mut xs) fresh_symbol(x) { xs.push(x); xs }
     pattern_symbols ::= any_symbol(x) { vec![x] }
@@ -207,7 +204,7 @@ pomelo! {
     prop_literals ::= prop_literals(mut xs) prop_literal(x) { xs.push(x); xs }
 
     // ⟨selector_dec⟩ ::= ( ⟨symbol⟩ ⟨sort⟩ )
-    selector_dec ::= LeftParen bound_symbol(x) sort(s) RightParen { (x, s) }
+    selector_dec ::= LeftParen fresh_symbol(x) sort(s) RightParen { (x, s) }
 
     selector_decs ::= selector_dec(x) { vec![x] }
     selector_decs ::= selector_decs(mut xs) selector_dec(x) { xs.push(x); xs }
@@ -318,7 +315,7 @@ pomelo! {
         }
     }
     //   ( define-sort ⟨symbol⟩ ( ⟨symbol⟩∗ ) ⟨sort⟩ )
-    command ::= LeftParen DefineSort fresh_symbol(x) LeftParen bound_symbols?(xs) RightParen sort(r) RightParen
+    command ::= LeftParen DefineSort fresh_symbol(x) LeftParen fresh_symbols?(xs) RightParen sort(r) RightParen
     {
         extra.0.visit_define_sort(x, xs.unwrap_or_else(Vec::new), r)?
     }
