@@ -71,7 +71,7 @@ pub trait SExprVisitor<Constant, Symbol, Keyword> {
 }
 
 #[derive(Debug, PartialEq, Eq, Clone, Hash, Serialize, Deserialize)]
-pub enum Index<Symbol> {
+pub enum Index<Symbol = crate::concrete::Symbol> {
     Numeral(Numeral),
     Symbol(Symbol),
 }
@@ -90,8 +90,9 @@ impl<S> Index<S> {
     }
 }
 
+/// Concrete identifier.
 #[derive(Debug, PartialEq, Eq, Clone, Hash, Serialize, Deserialize)]
-pub enum Identifier<Symbol> {
+pub enum Identifier<Symbol = crate::concrete::Symbol> {
     Simple {
         symbol: Symbol,
     },
@@ -148,7 +149,11 @@ pub trait QualIdentifierVisitor<Identifier, Sort> {
 }
 
 #[derive(Debug, PartialEq, Eq, Clone, Hash, Serialize, Deserialize)]
-pub enum AttributeValue<Constant, Symbol, SExpr> {
+pub enum AttributeValue<
+    Constant = crate::concrete::Constant,
+    Symbol = crate::concrete::Symbol,
+    SExpr = crate::concrete::SExpr,
+> {
     None,
     Constant(Constant),
     Symbol(Symbol),
@@ -266,7 +271,7 @@ impl<T1, T2> ConstructorDec<T1, T2> {
 }
 
 #[derive(Debug, PartialEq, Eq, Clone, Hash, Serialize, Deserialize)]
-pub struct DatatypeDec<Symbol, Sort> {
+pub struct DatatypeDec<Symbol = crate::concrete::Symbol, Sort = crate::concrete::Sort> {
     pub parameters: Vec<Symbol>,
     pub constructors: Vec<ConstructorDec<Symbol, Sort>>,
 }
@@ -303,7 +308,7 @@ impl<T1, T2> DatatypeDec<T1, T2> {
 }
 
 #[derive(Debug, PartialEq, Eq, Clone, Hash, Serialize, Deserialize)]
-pub struct FunctionDec<Symbol, Sort> {
+pub struct FunctionDec<Symbol = crate::concrete::Symbol, Sort = crate::concrete::Sort> {
     pub name: Symbol,
     pub parameters: Vec<(Symbol, Sort)>,
     pub result: Sort,
